/* --------------------------------------------------------------
                              pfs_7seg.h

     Header fuer Softwaremodul zur Ansteuerung des Padauk PFS154
     realisierten 7-Segmentdekoders.

     MCU      : AVR
     Compiler : avr-gcc

     zusaetzliche Hardware:
       PFS154 - 4 Digit 7-Segmentanzeige

     Wichtig: in die Datenleitung zum PFS-Controller muss
     ein Serienwiderstand von 4,7 kOhm geschaltet werden
     weil ansonsten der PFS Unfug macht (warum auch immer)

     21.06.2023   R. Seelig
   -------------------------------------------------------------- */

#ifndef in_pfs7seg
  #define in_pfs7seg

  #include <avr/io.h>
  #include <util/delay.h>
  #include "avr_gpio.h"

  #define set4dez_enable     1             // 1 : 4-stellige Dezimaleausgabe verfuegbar
                                           // 0 : nicht verfuegbar
  #define set2dez_enable     1             // 1 : 2-stellige Dezimalausgabe verfuegbar
                                           // 0 : nicht vefuegbar
  #define sethex_enable      1             // 1 : Hexadezimalausgabe verfuegbar
                                           // 0 : nicht verfuegbar

  /* --------------------------------------------------
       Pinanschluss, auf dem das 7-Segment Bitmuster
       gesendet wird
     -------------------------------------------------- */

  #define tx_init()   PD1_output_init()
  #define tx_set()    PD1_set()
  #define tx_clr()    PD1_clr()

  /* --------------------------------------------------
                           Variable
     -------------------------------------------------- */

  extern const uint8_t seg7bmp[];
  extern uint8_t seg7buffer[];

  /* --------------------------------------------------
                          Prototypen
     -------------------------------------------------- */
  void pfs7seg_init(void);
  void pfs7seg_sendbmp(void);
  #if (set4dez_enable == 1)
    void pfs7seg_set4dez(int16_t val, uint8_t komma_pos);
  #endif
  #if (set2dez_enable == 1)
    void pfs7seg_set2dez(int8_t val, uint8_t pos);
  #endif
  #if (sethex_enable == 1)
    void pfs7seg_sethex(uint16_t val);
  #endif

#endif
