/* ---------------------------------------------------------
                         pfs_7segdemo.c

     Demo zur Ansteuerung einer 4-Digit LED-Anzeige, die
     mit einer PFS154 MCU realisiert ist.

     MCU      : AVR
     Compiler : avr-gcc

     zusaetzliche Hardware:
       PFS154 - 4 Digit 7-Segmentanzeige

     Wichtig: in die Datenleitung zum PFS-Controller muss
     ein Serienwiderstand von 4,7 kOhm geschaltet werden
     weil ansonsten der PFS Unfug macht (warum auch immer)

     20.06.2023   R. Seelig
   --------------------------------------------------------- */

#include <avr/io.h>
#include <util/delay.h>

#include "avr_gpio.h"
#include "pfs_7seg.h"

#define delay     _delay_ms

uint8_t laufmust[] =
  {
    1,0,0,0, 0,1,0,0, 0,0,1,0, 0,0,0,1,
    0,0,0,2, 0,0,0,4,
    0,0,0,8, 0,0,8,0, 0,8,0,0, 8,0,0,0,
    0x10,0,0,0, 0x20,0,0,0
  };

/* --------------------------------------------------
                            main
   -------------------------------------------------- */
int main(void)
{
  int16_t i;
  uint8_t cx;

  pfs7seg_init();

  while(1)
  {

    // Lauflicht
    for (cx= 0; cx< 3; cx++)
    {
      for (i= 0; i< 48; i+= 4)
      {
        seg7buffer[3]= laufmust[i];
        seg7buffer[2]= laufmust[i+1];
        seg7buffer[1]= laufmust[i+2];
        seg7buffer[0]= laufmust[i+3];

        delay(50);
        pfs7seg_sendbmp();
      }
    }

    // eine hexadezimale Zahl anzeigen (3A4F)
    pfs7seg_sethex(0x3a4f);
    delay(1000);

    // eine Zahl mit Komma anzeigen (812,7(
    pfs7seg_set4dez(8127,1);
    delay(1000);

    // als Demo 25 Grad Celcius anzeigen

    seg7buffer[0]= seg7bmp[12];     // Bitmap 12 ist ein grosses "C"
    seg7buffer[1]= seg7bmp[17];     // Bitmap 17 ist ein hochgestelltes "o"
    seg7buffer[2]= 0x00;            // Anzeige dunkel
    seg7buffer[3]= 0x00;            // dto.

    pfs7seg_set2dez(25,3);           // zusaetzlich 25 an linker Stelle ausgeben
    delay(1000);

    // einen Dezimalzaehler anzeigen
    for (i= 2023; i< 2034; i++)
    {
      pfs7seg_set4dez(i,0);
      delay(250);
    }
  }
}
